from typing import Dict, Any

from django_scaffolding_tools.enums import ASTDataType
from django_scaffolding_tools.exceptions import DjangoParsingException, DSTException


def parse_for_django_classes(module: Dict[str, Any], raise_errors=False) -> Dict[str, any]:
    module_content = dict()
    django_classes = list()
    module_content['classes'] = django_classes

    for content in module['body']:
        if content.get('_type') == ASTDataType.CLASS:
            model = dict()
            model['name'] = content.get('name')
            model['attributes'] = list()
            for class_content in content['body']:
                if class_content.get('_type') == ASTDataType.ASSIGN:
                    field = dict()
                    # Name of the field
                    field['name'] = class_content['targets'][0]['id']
                    field['keywords'] = list()
                    field['arguments'] = list()
                    try:
                        field['data_type'] = get_field_data_type(class_content)
                    except KeyError as e:
                        error_message = f'Key error {e}. Model {model["name"]} attr {field["name"]}'
                        raise DjangoParsingException(error_message)

                    if field['data_type'] == 'ForeignKey':
                        first_argument = class_content['value']['args'][0]
                        if first_argument.get('_type') == ASTDataType.ATTRIBUTE:
                            if first_argument.get('attr') == 'AUTH_USER_MODEL':
                                field['arguments'].append({'name': 'fk_classname', 'value': 'User'})
                        elif first_argument.get('_type') == ASTDataType.CONSTANT:
                            fk_classname = first_argument['value']
                            field['arguments'].append({'name': 'fk_classname', 'value': fk_classname})
                        else:
                            try:
                                fk_classname = first_argument['id']
                                field['arguments'].append({'name': 'fk_classname', 'value': fk_classname})
                            except KeyError as e:
                                error_message = f'Error parsing field {field["name"]}'
                                raise DjangoParsingException(error_message)
                    try:
                        for keyword in class_content['value']['keywords']:
                            keyword_data = dict()
                            keyword_data['name'] = keyword['arg']
                            value_type = keyword['value']['_type']
                            keyword_data['value_type_TMP'] = value_type

                            keyword_value = None
                            if value_type == ASTDataType.CONSTANT:
                                keyword_value = keyword['value'].get('value')
                            elif value_type == ASTDataType.NAME:
                                keyword_value = keyword['value'].get('id')
                            elif value_type == ASTDataType.ATTRIBUTE:
                                keyword_value = keyword['value'].get('attr')
                            elif value_type == ASTDataType.CALL and keyword_data['name'] in ['help_text',
                                                                                             'verbose_name']:
                                if len(keyword['value'].get('args')) > 0:
                                    call_args_type = keyword['value']['args'][0]['_type']
                                    if call_args_type == ASTDataType.CONSTANT:
                                        keyword_value = keyword['value']['args'][0].get('value')
                            else:
                                if raise_errors:
                                    error_message = 'Unexpected parsing error'
                                    raise DjangoParsingException(error_message)

                            keyword_data['value'] = keyword_value
                            field['keywords'].append(keyword_data)
                    except KeyError:
                        print(f'Error with field {model["name"]}.{field["name"]} this field will be ignored.')
                    model['attributes'].append(field)
            django_classes.append(model)
    return module_content


def get_field_data_type(class_content, raise_errors=False):
    data_type = None
    if class_content['value']['_type'] == ASTDataType.CONSTANT:
        data_type = 'CONSTANT'
    elif class_content['value']['_type'] == ASTDataType.CALL:
        func_ = class_content['value']['func']
        data_type = func_.get('attr')
        if data_type is None:
            data_type = func_.get('id')
    elif class_content['value']['_type'] == ASTDataType.NAME:
        func_ = class_content['value']['func']
        data_type = func_.get('id')
    else:
        error_msg = f'Unsupported data type {class_content["value"]["_type"]}.'
        DjangoParsingException(error_msg)
    if data_type is None:
        if raise_errors:
            raise DjangoParsingException('Error')
        data_type = 'UNSUPPORTED'
    return data_type



from pathlib import Path
from pprint import pprint

from git import Repo, Commit
def get_commits():
    current_project = Path(__file__).parent.parent.parent.parent
    print(current_project)
    repo = Repo(current_project)
    commits = list(repo.iter_commits('master', max_count=5))
    cs =[{'name': str(c.author), 'email': c.author.email, 'msg': c.message, 'date': c.committed_date} for c in commits]
    return cs


if __name__ == '__main__':
    m_commits = get_commits()
    for m_commit in m_commits:
        pprint(m_commit)



"""
Base settings to build other settings files upon.
"""
from enum import Enum
from pathlib import Path

import environ

ROOT_DIR = Path(__file__).resolve(strict=True).parent.parent
# ec_d_local_payment_collector/
APPS_DIR = ROOT_DIR / "ec_d_local_payment_collector"
TEST_OUTPUT_PATH = ROOT_DIR / 'output'

env = environ.Env()
TEST_APP_SERVERS = 'example.servers'
TEST_APP_PEOPLE = 'example.people'

READ_DOT_ENV_FILE = env.bool("DJANGO_READ_DOT_ENV_FILE", default=False)
if READ_DOT_ENV_FILE:
    # OS environment variables take precedence over variables from .env
    env.read_env(str(ROOT_DIR / ".env"))

# GENERAL
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#debug
DEBUG = env.bool("DJANGO_DEBUG", True)
# Local time zone. Choices are
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# though not all of them may be available with every OS.
# In Windows, this must be set to your system time zone.
TIME_ZONE = "America/Panama"
# https://docs.djangoproject.com/en/dev/ref/settings/#language-code
LANGUAGE_CODE = "en-us"
# https://docs.djangoproject.com/en/dev/ref/settings/#site-id
SITE_ID = 1
# https://docs.djangoproject.com/en/dev/ref/settings/#use-i18n
USE_I18N = True
# https://docs.djangoproject.com/en/dev/ref/settings/#use-l10n
USE_L10N = True
# https://docs.djangoproject.com/en/dev/ref/settings/#use-tz
USE_TZ = True
# https://docs.djangoproject.com/en/dev/ref/settings/#locale-paths
LOCALE_PATHS = [str(ROOT_DIR / "locale")]

# DATABASES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#databases
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': ROOT_DIR / 'django-scaffolding-tools.sqlite3'
    }
}
DATABASES["default"]["ATOMIC_REQUESTS"] = True
# https://docs.djangoproject.com/en/stable/ref/settings/#std:setting-DEFAULT_AUTO_FIELD
DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

# URLS
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#root-urlconf
ROOT_URLCONF = "config.urls"
# https://docs.djangoproject.com/en/dev/ref/settings/#wsgi-application
WSGI_APPLICATION = "config.wsgi.application"

# APPS
# ------------------------------------------------------------------------------
DJANGO_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    # "django.contrib.humanize", # Handy template tags
    "django.contrib.admin",
    "django.forms",
]
THIRD_PARTY_APPS = [
    "crispy_forms",
    "crispy_bootstrap5",
    "allauth",
    "allauth.account",
    "allauth.socialaccount",
    "django_celery_beat",
    "rest_framework",
    "rest_framework.authtoken",
    "corsheaders",
    "drf_spectacular",
    'django_celery_results'
]

LOCAL_APPS = [
    "ec_d_local_payment_collector.users",
    "ec_d_local_payment_collector.ec_d_local",
    "ec_d_local_payment_collector.service_configuration",
    TEST_APP_SERVERS,
    TEST_APP_PEOPLE,
]
# https://docs.djangoproject.com/en/dev/ref/settings/#installed-apps
INSTALLED_APPS = DJANGO_APPS + THIRD_PARTY_APPS + LOCAL_APPS

# MIGRATIONS
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#migration-modules
MIGRATION_MODULES = {"sites": "ec_d_local_payment_collector.contrib.sites.migrations"}

# AUTHENTICATION
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#authentication-backends
AUTHENTICATION_BACKENDS = [
    "django.contrib.auth.backends.ModelBackend",
    "allauth.account.auth_backends.AuthenticationBackend",
]
# https://docs.djangoproject.com/en/dev/ref/settings/#auth-user-model
AUTH_USER_MODEL = "users.User"
# https://docs.djangoproject.com/en/dev/ref/settings/#login-redirect-url
LOGIN_REDIRECT_URL = "users:redirect"
# https://docs.djangoproject.com/en/dev/ref/settings/#login-url
LOGIN_URL = "account_login"

# PASSWORDS
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#password-hashers
PASSWORD_HASHERS = [
    # https://docs.djangoproject.com/en/dev/topics/auth/passwords/#using-argon2-with-django
    "django.contrib.auth.hashers.Argon2PasswordHasher",
    "django.contrib.auth.hashers.PBKDF2PasswordHasher",
    "django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher",
    "django.contrib.auth.hashers.BCryptSHA256PasswordHasher",
]
# https://docs.djangoproject.com/en/dev/ref/settings/#auth-password-validators
AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator"
    },
    {"NAME": "django.contrib.auth.password_validation.MinimumLengthValidator"},
    {"NAME": "django.contrib.auth.password_validation.CommonPasswordValidator"},
    {"NAME": "django.contrib.auth.password_validation.NumericPasswordValidator"},
]

# MIDDLEWARE
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#middleware
MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "corsheaders.middleware.CorsMiddleware",
    "whitenoise.middleware.WhiteNoiseMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.common.BrokenLinkEmailsMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

# STATIC
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#static-root
STATIC_ROOT = str(ROOT_DIR / "staticfiles")
# https://docs.djangoproject.com/en/dev/ref/settings/#static-url
STATIC_URL = "/static/"
# https://docs.djangoproject.com/en/dev/ref/contrib/staticfiles/#std:setting-STATICFILES_DIRS
STATICFILES_DIRS = [str(APPS_DIR / "static")]
# https://docs.djangoproject.com/en/dev/ref/contrib/staticfiles/#staticfiles-finders
STATICFILES_FINDERS = [
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
]

# MEDIA
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#media-root
MEDIA_ROOT = str(APPS_DIR / "media")
# https://docs.djangoproject.com/en/dev/ref/settings/#media-url
MEDIA_URL = "/media/"

# TEMPLATES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#templates
TEMPLATES = [
    {
        # https://docs.djangoproject.com/en/dev/ref/settings/#std:setting-TEMPLATES-BACKEND
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        # https://docs.djangoproject.com/en/dev/ref/settings/#dirs
        "DIRS": [str(APPS_DIR / "templates")],
        # https://docs.djangoproject.com/en/dev/ref/settings/#app-dirs
        "APP_DIRS": True,
        "OPTIONS": {
            # https://docs.djangoproject.com/en/dev/ref/settings/#template-context-processors
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    }
]

# https://docs.djangoproject.com/en/dev/ref/settings/#form-renderer
FORM_RENDERER = "django.forms.renderers.TemplatesSetting"

# http://django-crispy-forms.readthedocs.io/en/latest/install.html#template-packs
CRISPY_TEMPLATE_PACK = "bootstrap5"
CRISPY_ALLOWED_TEMPLATE_PACKS = "bootstrap5"

# FIXTURES
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#fixture-dirs
FIXTURE_DIRS = (str(APPS_DIR / "fixtures"),)

# SECURITY
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#session-cookie-httponly
SESSION_COOKIE_HTTPONLY = True
# https://docs.djangoproject.com/en/dev/ref/settings/#csrf-cookie-httponly
CSRF_COOKIE_HTTPONLY = True
# https://docs.djangoproject.com/en/dev/ref/settings/#secure-browser-xss-filter
SECURE_BROWSER_XSS_FILTER = True
# https://docs.djangoproject.com/en/dev/ref/settings/#x-frame-options
X_FRAME_OPTIONS = "DENY"

# EMAIL
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#email-backend
EMAIL_BACKEND = env(
    "DJANGO_EMAIL_BACKEND",
    default="django.core.mail.backends.smtp.EmailBackend",
)
# https://docs.djangoproject.com/en/dev/ref/settings/#email-timeout
EMAIL_TIMEOUT = 5

# ADMIN
# ------------------------------------------------------------------------------
# Django Admin URL.
ADMIN_URL = "admin/"
# https://docs.djangoproject.com/en/dev/ref/settings/#admins
ADMINS = [("""Luis C. Berrocal""", "luis.berrocal@payjoy.com")]
# https://docs.djangoproject.com/en/dev/ref/settings/#managers
MANAGERS = ADMINS

# LOGGING
# ------------------------------------------------------------------------------
# https://docs.djangoproject.com/en/dev/ref/settings/#logging
# See https://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "formatters": {
        "verbose": {
            "format": "%(levelname)s %(asctime)s %(module)s "
                      "%(process)d %(thread)d %(message)s"
        }
    },
    "handlers": {
        "console": {
            "level": "DEBUG",
            "class": "logging.StreamHandler",
            "formatter": "verbose",
        }
    },
    "loggers": {
        'ec_d_local_payment_collector': {
            "level": "DEBUG",
            "handlers": ['console'],
            "propagate": False
        },
    },
    "root": {"level": "INFO", "handlers": ["console"]},
}

# django-rest-framework
# -------------------------------------------------------------------------------
# django-rest-framework - https://www.django-rest-framework.org/api-guide/settings/
REST_FRAMEWORK = {
    "DEFAULT_AUTHENTICATION_CLASSES": (
        "rest_framework.authentication.SessionAuthentication",
        "rest_framework.authentication.TokenAuthentication",
    ),
    "DEFAULT_PERMISSION_CLASSES": ("rest_framework.permissions.IsAuthenticated",),
    "DEFAULT_SCHEMA_CLASS": "drf_spectacular.openapi.AutoSchema",
    'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.LimitOffsetPagination',
    'PAGE_SIZE': 10
}

# django-cors-headers - https://github.com/adamchainz/django-cors-headers#setup
CORS_URLS_REGEX = r"^/api/.*$"


# Your stuff...
# ------------------------------------------------------------------------------

PERMISSION_GROUPS = {
    'payment_read': {
        'name': 'PaymentReadGroup',
        'description': 'Reading payments is allowed to this group'
    },
    'payment_callback': {
        'name': 'PaymentCallbackGroup',
        'description': 'Calling callback url is allowed to this group'
    },
    'payment_update': {
        'name': 'PaymentUpdateGroup',
        'description': 'Updating payments is allowed to this group'
    },
    'payment_create': {
        'name': 'PaymentCreateGroup',
        'description': 'Creating payments is allowed to this group'
    },
}




from datetime import datetime
from pathlib import Path
from typing import Dict, Any

from docxtpl import DocxTemplate

from django_scaffolding_tools._experimental.cover_letters.utils import run_commands


def convert_docx_to_pdf(docx_file: Path, output_folder: Path):
    command = ['libreoffice', '--headless', '--convert-to', 'pdf', '--outdir', f'{output_folder}',
               f'{docx_file}']
    result, errors = run_commands(command)

    return result, errors


def write_docx_cover_letter(template_file: Path, context: Dict[str, Any], output_file: Path):
    # Open our master template
    doc = DocxTemplate(template_file)
    # Load them up
    doc.render(context)
    # Save the file with personalized filename
    doc.save(output_file)


if __name__ == '__main__':
    root_folder = Path(__file__).parent.parent.parent.parent
    fixtures_folder = root_folder / 'tests' / 'fixtures'
    out_folder = root_folder / 'output'

    template = fixtures_folder / '_experimental' / 'Cover Letter Template.docx'
    today = datetime.today()
    context = {'today': today.strftime('%b %M %Y'), 'position_name': 'Jedi', 'company_name': 'Jedi Order'}

    cover_letter = out_folder / f'{today.strftime("%Y%m%d")}_cover_{context["company_name"]}_{context["position_name"]}.pdf'
    cover_letter.unlink(missing_ok=True)

    write_docx_cover_letter(template, context, cover_letter)



import json

from django_scaffolding_tools._experimental.reqs_utilities.db import RequirementDatabase
from django_scaffolding_tools.utils.core import quick_write


class TestRequirementsDatabase:

    def test_r(self, output_folder, fixtures_folder):
        local_file = fixtures_folder / '_experimental' / 'requirements'
        json_db_file = output_folder / 'json_db.json'
        parsed = output_folder / '_local.json'
        db = RequirementDatabase(json_db_file)
        reqs = db.get_from_requirements_folder(local_file)
        reqs_dict = dict()
        for name, req in reqs.items():
            reqs_dict[name] = req.dict()
        with open(parsed, 'w') as f:
            json.dump(reqs_dict, f, indent=4, default=str)

    def test_get(self, fixtures_folder):
        json_db_file = fixtures_folder / '_experimental' / 'req_db.json'
        db = RequirementDatabase(json_db_file)
        req = db.get('django')
        assert req is not None
        assert req.approved_version == '3.2.16'

    def test_update_db(self, fixtures_folder):
        json_db_file = fixtures_folder / '_experimental' / 'req_db.json'
        db = RequirementDatabase(json_db_file)
        db.update_db()

    def test_add(self, fixtures_folder):
        json_db_file = fixtures_folder / '_experimental' / 'req_db.json'
        db = RequirementDatabase(json_db_file)
        db.add('Werkzeug', environment='local')

    def test_greatest(self):
        version1 = "4.1.1"
        version_info1 = tuple(
            [
                int(num) if num.isdigit() else num
                for num in version1.replace("-", ".", 1).split(".")
            ]
        )
        version2 = "3.2.16"
        version_info2 = tuple(
            [
                int(num) if num.isdigit() else num
                for num in version2.replace("-", ".", 1).split(".")
            ]
        )
        assert version_info1 > version_info2




from typing import List, Optional, Union

from pydantic import BaseModel


class Keyword(BaseModel):
    name: str
    value: Union[int, str]


class Serializer(BaseModel):
    field_type: str
    keywords: List[Keyword]


class Attribute(BaseModel):
    name: str
    value: str
    supported: bool
    native: bool
    alias: Optional[str] = None
    data_type: str
    length: int
    serializer: Serializer


class Model(BaseModel):
    name: str
    level: int
    attributes: List[Attribute]



import json
import re
from datetime import datetime
from pathlib import Path
from typing import Optional, Dict

import requests
from johnnydep.pipper import get_versions
from pydantic import ValidationError

from django_scaffolding_tools._experimental.reqs_utilities.models import RecommendedRequirement


class RequirementDatabase:


    def __init__(self, source_file: Path):
        self.source_file = source_file
        self.regexp = re.compile(r'(?P<lib_name>[\w_\-]+)==(?P<version>[\w\.\-]+)\s*#?(?P<comment>.*)')
        self.database = dict()
        self.load_db(self.source_file)

    def load_db(self, source_file: Path):
        with open(source_file, 'r') as j_file:
            data = json.load(j_file)
        for name, req_dict in data.items():
            try:
                self.database[name] = RecommendedRequirement(**req_dict)
            except ValidationError as e:
               error_message = f'Invalid {name} library content. {e}'
               raise Exception(error_message)
    def get(self, name: str) -> RecommendedRequirement:
        return self.database.get(name)

    def add(self, name: str, environment: Optional[str], version: Optional[str] = None,
            commit: bool = True) -> RecommendedRequirement:
        if self.get(name) is not None:
            raise Exception(f'Library {name} already exists use update.')
        versions = get_versions(name)
        if len(versions) == 0:
            raise Exception(f'Library {name} not found')
        latest_version = versions[-1]
        approved_version = version if version is not None else latest_version
        recommended = RecommendedRequirement(name=name, latest_version=latest_version,
                                             approved_version=approved_version,
                                             environment=environment)
        info = self._download_info(name, recommended.approved_version)
        recommended.home_page = info['home_page']
        recommended.license = info['license']
        recommended.last_updated = datetime.now()
        self.database[name] = recommended
        if commit:
            self.save()
        return recommended

    def update(self, name: str) -> RecommendedRequirement:
        req = self.get(name)
        if req is None:
            raise Exception(f'Requirement {name} does not exist.')
        versions = get_versions(name)
        if len(versions) == 0:
            raise Exception(f'Library {name} not found')
        latest_version = versions[-1]
        req.approved_version = latest_version
        req.latest_version = latest_version
        req.last_updated = datetime.now()
        self.database[name] = req
        self.save()
        return req

    def update_db(self, commit: bool = True):
        for name, req in self.database.items():
            info = self._download_info(name, req.approved_version)
            req.home_page = info['home_page']
            req.license = info['license']
            req.last_updated = datetime.now()
        if commit:
            self.save()

    def _download_info(self, name: str, version: str):
        url = f'https://pypi.org/pypi/{name}/{version}/json'
        response = requests.get(url)
        info = dict()
        if response.status_code == 200:
            data = response.json()
            home_page = data['info']['home_page']
            lic = data['info']['license']
            info = {'home_page': home_page, 'license': lic}
        return info

    def save(self):
        tmp = dict()
        for name, req in self.database.items():
            tmp[name] = req.dict()
        with open(self.source_file, 'w') as f:
            json.dump(tmp, f, indent=4, default=str)
    def get_from_requirements_folder(self, folder: Path):
        req_files = folder.glob('**/*.txt')
        global_req = dict()
        for req_file in req_files:
            reqs = self.get_from_requirement_file(req_file)
            global_req.update(reqs)
        return global_req

    def get_from_requirement_file(self, req_file: Path) -> Dict[str, RecommendedRequirement]:
        with open(req_file, 'r') as r_file:
            lines = r_file.readlines()
        parsed_requirements = dict()
        for i, line in enumerate(lines, 1):
            match = self.regexp.match(line)
            if match:
                lib_name = match.group('lib_name')
                versions = get_versions(lib_name)
                if len(versions) == 0:
                    raise Exception(f'Library {lib_name} not found')
                latest_version = versions[-1]
                recommended = RecommendedRequirement(name=lib_name, latest_version=latest_version,
                                                     approved_version=match.group('version'),
                                                     environment=req_file.stem)
                parsed_requirements[lib_name] = recommended
        return parsed_requirements

def check_for_new_requirements(db: RequirementDatabase):
    for name, req in db.database.items():
        info = db._download_info(name, req.approved_version)
        versions = get_versions(name)
        latest = versions[-1]
        if latest == req.approved_version:
            continue
        print(f'{name} {latest}')
        print(f'Approved {req.approved_version} Latest{req.latest_version}')
        print(info)
        action = input('Update approved?')
        if action.upper() =='Y':
            req.approved_version = latest
            req.latest_version = latest
            req.last_updated = datetime.now()
            db.save()

if __name__ == '__main__':
    db_file = Path(__file__).parent / 'req_db.json'
    db = RequirementDatabase(db_file)
    check_for_new_requirements(db)



#!/usr/bin/env python

"""Tests for `django_scaffolding_tools` package."""

import pytest
from click.testing import CliRunner

from django_scaffolding_tools import cli


@pytest.fixture
def response():
    """Sample pytest fixture.

    See more at: http://doc.pytest.org/en/latest/fixture.html
    """
    # import requests
    # return requests.get('https://github.com/audreyr/cookiecutter-pypackage')


def test_command_line_interface():
    """Test the CLI."""
    runner = CliRunner()
    result = runner.invoke(cli.main)
    assert result.exit_code == 0
    assert 'django_scaffolding_tools.cli.main' in result.output
    help_result = runner.invoke(cli.main, ['--help'])
    assert help_result.exit_code == 0
    assert '--help  Show this message and exit.' in help_result.output


def test_cmd_json_to_ser(output_folder):
    out_file = output_folder / '__serializers.py'
    out_file.unlink(missing_ok=True)

    runner = CliRunner()
    help_result = runner.invoke(cli.main, ['J2SER', '--source-file', './fixtures/json_data.json', '--output-folder',
                                           output_folder])

    assert help_result.exit_code == 0
    print(help_result.output)
    assert out_file.exists()



from autoslug import AutoSlugField
from django.conf import settings

from django.db.models import JSONField
from django.db import models, IntegrityError
from django.utils.translation import ugettext_lazy as _
from model_utils.models import TimeStampedModel

from .exceptions import ClinicException
from ..core.models import Human, Auditable
# Create your models here.
from ..patients.models import Patient


def build_slug(physician):
    return '{} {}'.format(physician.last_name, physician.first_name)


def upload_logo(instance, filename):
    extension = filename.split('.')[-1]
    return f'{instance.folder}/{instance.slug}-logo.{extension}'


def get_default_discount_config():
    json_data = settings.EMR_PRACTICE['DEFAULT_DISCOUNT_CONFIG']
    return json_data


class Clinic(models.Model):
    name = models.CharField(_('Name'), max_length=100)
    short_name = models.CharField(_('Short name'), max_length=20, null=True, blank=True)
    slug = AutoSlugField(populate_from='name', unique=True, always_update=True)
    folder = AutoSlugField(populate_from='name', unique=True)
    owner = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.PROTECT, verbose_name=_('Owner'),
                              related_name='owned_clinics')
    logo = models.FileField(upload_to=upload_logo, verbose_name=_('Clinic logo'), null=True, blank=True)
    default_hospital = models.ForeignKey('CareTakingFacility', verbose_name=_('Default Hospital'), null=True,
                                         blank=True, on_delete=models.SET_NULL)
    discount_config = JSONField(_('Discount configuration'), null=True, blank=True, default=get_default_discount_config)
    settings = JSONField(_('Settings'), null=True, blank=True, default=get_default_discount_config)

    def __str__(self):
        return self.name

    class Meta:
        ordering = ('name',)

    def add_member(self, clinic_member):
        try:
            return ClinicMembership.objects.create(clinic_member=clinic_member, clinic=self)
        except IntegrityError:
            msg = _('{} is already a member of {}'.format(clinic_member, self))
            raise ClinicException(msg)

    def balance(self):
        from ..finance.models import Invoice
        return Invoice.objects.outstanding_balance(self)


class Company(models.Model):
    name = models.CharField(_('Name'), max_length=100)
    short_name = models.CharField(_('Short name'), max_length=20, null=True, blank=True)
    national_id = models.CharField(_('National id'), max_length=20, null=True, blank=True)
    verification_digit = models.CharField(_('Verification digit'), max_length=4, null=True, blank=True)
    clinic = models.ForeignKey(Clinic, verbose_name=_('Clinic'), related_name='companies', on_delete=models.PROTECT)
    order = models.IntegerField(_('Order'), default=1)
    settings = JSONField(_('Settings'), null=True, blank=True, default=get_default_discount_config)
    slug = AutoSlugField(populate_from='name', unique=True, always_update=True)
    folder = AutoSlugField(populate_from='name', unique=True)
    logo = models.ImageField(upload_to=upload_logo, verbose_name=_('Company logo'), null=True, blank=True)

    class Meta:
        ordering = ('order', 'name')

    def __str__(self):
        return self.name


class ClinicMembership(models.Model):
    clinic_member = models.ForeignKey('clinics.ClinicMember', verbose_name=_('Clinic member'), on_delete=models.PROTECT)
    clinic = models.ForeignKey(Clinic, verbose_name=_('Clinic'), on_delete=models.PROTECT)
    settings = JSONField(_('Settings'), null=True, blank=True, default=get_default_discount_config)

    class Meta:
        unique_together = ('clinic_member', 'clinic')


class CareTakingFacility(models.Model):
    name = models.CharField(max_length=120, verbose_name=_('Name'))
    short_name = models.CharField(max_length=60, verbose_name=_('Short name'), null=True, blank=True)
    address = models.TextField(verbose_name=_('Address'))
    google_map_link = models.URLField(null=True, blank=True)

    class Meta:
        ordering = ('name',)

    def __str__(self):
        return self.name


class ClinicMember(Human):
    PHYSICIAN_ROLE = 'PHYSICIAN'
    ASSISTANT_ROLE = 'ASSISTANT'
    ROLES = (
        (PHYSICIAN_ROLE, _('Physician')),
        (ASSISTANT_ROLE, _('Assistant'))
    )
    user = models.OneToOneField(settings.AUTH_USER_MODEL, on_delete=models.SET_NULL, null=True, blank=True,
                                related_name='clinic_member')
    slug = AutoSlugField(populate_from=build_slug)
    role = models.CharField(_('role'), choices=ROLES, default=PHYSICIAN_ROLE, max_length=10)
    clinics = models.ManyToManyField(Clinic, through=ClinicMembership, related_name='members')

    def __str__(self):
        return '{}, {}'.format(self.last_name, self.first_name)

    class Meta:
        ordering = ('last_name', 'first_name')

    def patients(self):
        clinics = ClinicMembership.objects.filter(clinic_member=self).values_list('clinic__id', flat=True)
        return Patient.objects.filter(clinic__in=clinics)


class UniversalBillingCode(TimeStampedModel):
    """
    Universal code for billing and insurance purposes. Billing codes will have a source CPT
    (https://coder.aapc.com/cpt-codes/)
    or Local which in the begining will only include Other.
    """
    CPT_SOURCE = 'CPT'
    LOCAL_SOURCE = 'LOCAL'
    SOURCES = (
        (CPT_SOURCE, _('CPT')),
        (LOCAL_SOURCE, _('Local'))
    )
    source = models.CharField(_('Source'), max_length=8, choices=SOURCES, default=CPT_SOURCE)
    code = models.CharField(_('Code'), max_length=10, unique=True)
    description = models.CharField(_('Description'), max_length=100)

    def __str__(self):
        return '{} - {}'.format(self.code, self.description)

    class Meta:
        ordering = ('code',)


class Diagnostic(Auditable, TimeStampedModel):
    PRIMARY_TYPE = 'PRIM'
    SECONDARY_TYPE = 'SEC'
    TYPE_CHOICES = (
        (PRIMARY_TYPE, _('Primary')),
        (SECONDARY_TYPE, _('Secondary')),
    )
    type = models.CharField(_('Type'), max_length=10, choices=TYPE_CHOICES, default=PRIMARY_TYPE)
    code = models.CharField(_('Code'), max_length=15, unique=True)
    description = models.CharField(_('Description'), max_length=150)

    def __str__(self):
        return '{} - {}'.format(self.code, self.description)

    class Meta:
        ordering = ('description',)


class Procedure(Auditable, TimeStampedModel):
    code = models.CharField(_('Code'), max_length=15, unique=True)
    description = models.CharField(_('Description'), max_length=150)

    def __str__(self):
        return '{} - {}'.format(self.code, self.description)

    class Meta:
        ordering = ('description',)


class ClinicDiagnostic(Auditable, TimeStampedModel):
    type = models.CharField(_('Type'), max_length=10, null=True, choices=Diagnostic.TYPE_CHOICES)
    clinic = models.ForeignKey(Clinic, on_delete=models.PROTECT, verbose_name=_('Clinic'),
                               related_name='clinic_diagnostics')
    diagnostic = models.ForeignKey(Diagnostic, on_delete=models.PROTECT, verbose_name=_('Diagnostic'),
                                   related_name='clinic_diagnostics')

    class Meta:
        ordering = ('clinic', 'diagnostic__code')

    def __str__(self):
        if self.type is None:
            return '{} - {}'.format(self.diagnostic.code, self.diagnostic.description)
        else:
            return '{} ({}) - {}'.format(self.type, self.diagnostic.code, self.diagnostic.description)


class ClinicProcedure(Auditable, TimeStampedModel):
    clinic = models.ForeignKey(Clinic, on_delete=models.PROTECT, verbose_name=_('Clinic'),
                               related_name='clinic_procedures')
    procedure = models.ForeignKey(Procedure, on_delete=models.PROTECT, verbose_name=_('Procedure'),
                                  related_name='clinic_procedures')

    class Meta:
        ordering = ('clinic', 'procedure__code')
        unique_together = ('clinic', 'procedure')

    def __str__(self):
        return '{} - {}'.format(self.procedure.code, self.procedure.description)



import re
from abc import ABC, abstractmethod
from typing import Dict, Any, List, Optional

from django_scaffolding_tools.django.utils import get_max_length, get_decimal_info


class ModelFieldHandler(ABC):
    @abstractmethod
    def set_next(self, handler: 'ModelFieldHandler') -> 'ModelFieldHandler':
        pass

    @abstractmethod
    def handle(self, field_data: Dict[str, Any]) -> Dict[str, Any] | None:
        pass


class AbstractModelFieldHandler(ModelFieldHandler):
    _next_handler: ModelFieldHandler = None
    field = None
    _next_handlers = list()
    _current_index = 0

    def set_next(self, handler: 'ModelFieldHandler') -> 'ModelFieldHandler':
        self._next_handler = handler
        return handler

    def handle(self, field_data: Dict[str, Any]) -> Dict[str, Any] | None:
        if self._next_handler is not None:
            return self._next_handler.handle(field_data)
        return None

    def __iter__(self):
        self._next_handlers = list()
        self._current_index = 0
        my_next = self._next_handler
        self._next_handlers.append(my_next)
        while True:
            my_next = my_next._next_handler
            if my_next is None:
                break
            self._next_handlers.append(my_next)
        for handler in self._next_handlers:
            yield handler

    def __len__(self):
        self.__iter__()
        return len(self._next_handlers)


class DateTimeFieldHandler(AbstractModelFieldHandler):
    field = 'DateTimeField'

    def handle(self, field_data: Dict[str, Any]) -> Dict[str, Any] | None:
        if field_data['data_type'] == self.field:
            field_data['factory_field'] = 'LazyAttribute(lambda x: faker.date_time_between(start_date="-1y", ' \
                                          'end_date="now", tzinfo=timezone(settings.TIME_ZONE)))'
            return field_data
        else:
            return super().handle(field_data)


class DateFieldHandler(AbstractModelFieldHandler):
    field = 'DateField'

    def handle(self, field_data: Dict[str, Any]) -> Dict[str, Any] | None:
        if field_data['data_type'] == self.field:
            field_data['factory_field'] = 'LazyAttribute(lambda x: faker.date_time_between(start_date="-1y", ' \
                                          'end_date="now", tzinfo=timezone(settings.TIME_ZONE)).date())'
            return field_data
        else:
            return super().handle(field_data)


class ForeignKeyFieldHandler(AbstractModelFieldHandler):
    field = 'ForeignKey'

    def handle(self, field_data: Dict[str, Any]) -> Dict[str, Any] | None:
        if field_data['data_type'] == self.field:
            class_name = field_data['arguments'][0]['value']  # FIXME very flaky
            value = f'SubFactory({class_name}Factory)'
            field_data['factory_field'] = value
            return field_data
        else:
            return super().handle(field_data)


class IntegerFieldHandler(AbstractModelFieldHandler):
    field = 'IntegerField'

    def __init__(self):
        regexp_str = r'.*(datetime|timestamp|time).*'
        self.regexp = re.compile(regexp_str)

    def handle(self, field_data: Dict[str, Any]) -> Dict[str, Any] | None:
        if field_data['data_type'] == self.field:
            match = self.regexp.match(field_data['name'])
            if match is None:
                value = 'LazyAttribute(lambda o: randint(1, 100))'
            else:
                value = 'LazyAttribute(lambda x: faker.date_time_between(start_date="-1y", ' \
                        'end_date="now", tzinfo=timezone(settings.TIME_ZONE)).timestamp())'
            field_data['factory_field'] = value
            return field_data
        else:
            return super().handle(field_data)


class CharFieldHandler(AbstractModelFieldHandler):
    field = 'CharField'

    def __init__(self):
        regexp_str = r'.*(id|key|number).*'
        self.regexp = re.compile(regexp_str)

    def handle(self, field_data: Dict[str, Any]) -> Dict[str, Any] | None:
        if field_data['data_type'] == self.field:
            max_length = get_max_length(field_data)
            match = self.regexp.match(field_data['name'])
            if match is None:
                value = f'LazyAttribute(lambda x: FuzzyText(length={max_length}, chars=string.ascii_uppercase).fuzz())'
            else:
                value = f'LazyAttribute(lambda x: FuzzyText(length={max_length}, chars=string.digits).fuzz())'
            field_data['factory_field'] = value
            return field_data
        else:
            return super().handle(field_data)


class DecimalFieldHandler(AbstractModelFieldHandler):
    field = 'DecimalField'

    def __init__(self, min_value: str = '25.00', max_value: str = '500.00'):
        self.min_value = min_value
        self.max_value = max_value

    def handle(self, field_data: Dict[str, Any]) -> Dict[str, Any] | None:
        if field_data['data_type'] == self.field:
            max_digits, decimal_places = get_decimal_info(field_data)
            value = f'LazyAttribute(lambda x: faker.pydecimal(left_digits={max_digits - decimal_places}, ' \
                    f'right_digits={decimal_places}, ' \
                    f'positive=True, min_value={self.min_value}, max_value={self.max_value}))'
            field_data['factory_field'] = value
            return field_data
        else:
            return super().handle(field_data)


class BooleanFieldHandler(AbstractModelFieldHandler):
    field = 'BooleanField'

    def handle(self, field_data: Dict[str, Any]) -> Dict[str, Any] | None:
        if field_data['data_type'] == self.field:
            value = 'Iterator([True, False])'
            field_data['factory_field'] = value
            return field_data
        else:
            return super().handle(field_data)


class DateTimeCharFieldHandler(AbstractModelFieldHandler):
    field = 'CharField'

    def __init__(self):
        regexp_str = r'.*(datetime|date|time).*'
        self.regexp = re.compile(regexp_str)
        self.date_format = '%Y-%m-%dT%H:%M:%S%z'

    def handle(self, field_data: Dict[str, Any]) -> Dict[str, Any] | None:
        if field_data['data_type'] == self.field:
            match = self.regexp.match(field_data['name'])
            if match is not None:
                value = f'LazyAttribute(lambda x: faker.date_time_between(start_date="-1y", ' \
                        f'end_date="now", tzinfo=timezone(settings.TIME_ZONE)).strftime({self.date_format}))'
            else:
                return super().handle(field_data)
            field_data['factory_field'] = value
            return field_data
        else:
            return super().handle(field_data)


class EmailFieldHandler(AbstractModelFieldHandler):
    field = 'EmailField'

    def handle(self, field_data: Dict[str, Any]) -> Dict[str, Any] | None:
        if field_data['data_type'] == self.field:
            field_data['factory_field'] = f'LazyAttribute(lambda x: faker.ascii_free_email())'
            return field_data
        else:
            return super().handle(field_data)


class JSONFieldHandler(AbstractModelFieldHandler):
    field = 'JSONField'

    def handle(self, field_data: Dict[str, Any]) -> Dict[str, Any] | None:
        if field_data['data_type'] == self.field:
            field_data['factory_field'] = f'LazyAttribute(lambda x: faker.pydict(5, value_types=[str, int, float]))'
            return field_data
        else:
            return super().handle(field_data)



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()


with open("requirements.txt") as f:
    requirements = f.read().splitlines()

test_requirements = ['pytest>=3', ]

setup(
    author="Luis C. Berrocal",
    author_email='luis.berrocal.1942@gmail.com',
    python_requires='>=3.8',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
    description="Django tools for quick scaffolding.",
    entry_points={
        'console_scripts': [
            'django_scaffolding_tools=django_scaffolding_tools.cli:main',
            'django_gen=django_scaffolding_tools.django.cli:main',
        ],
    },
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='django_scaffolding_tools',
    name='django_scaffolding_tools',
    packages=find_packages(include=['django_scaffolding_tools', 'django_scaffolding_tools.*']),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/luiscberrocal/django_scaffolding_tools',
    version='0.10.0',
    zip_safe=False,
    package_data={'': ['templates/*.j2']},
)



from enum import Enum
from typing import List


class NativeDataType(str, Enum):
    INTEGER = 'int'
    STRING = 'str'
    FLOAT = 'float'
    DATETIME = 'datetime'
    DATE = 'date'

    @classmethod
    def to_list(cls) -> List[str]:
        code_list = [x.value for x in cls]
        return code_list


class PatternType(str, Enum):
    EMAIL = 'email_pattern'
    URL = 'url_pattern'
    DATETIME = 'datetime_pattern'
    DATE = 'date_pattern'


class ASTDataType(str, Enum):
    CLASS = 'ClassDef'
    ASSIGN = 'Assign'
    NAME = 'Name'
    CONSTANT = 'Constant'
    ATTRIBUTE = 'Attribute'
    CALL = 'Call'


class CommandType(str, Enum):
    JSON_TO_SERIALIZER = 'J2SER'
    JSON_TO_DJANGO_MODEL = 'J2DMOD'



from datetime import datetime
from typing import Optional

from pydantic import BaseModel, Field, HttpUrl


def convert_datetime_to_iso_8601_with_z_suffix(dt: datetime) -> str:
    return dt.strftime('%Y-%m-%dT%H:%M:%SZ')

def convert_version_to_tuples(version:str):
    version_info = tuple(
        [
            int(num) if num.isdigit() else num
            for num in version.replace("-", ".", 1).split(".")
        ]
    )
    return version_info


class RecommendedRequirement(BaseModel):
    name: str
    latest_version: str
    approved_version: str
    group: Optional[str]
    environment: Optional[str]
    last_updated: datetime = Field(default=datetime.now())
    home_page: Optional[HttpUrl]
    license: Optional[str]

    def to_req_line(self) -> str:
        if self.home_page is None:
            line = f'{self.name}=={self.approved_version}'
        else:
            line = f'{self.name}=={self.approved_version} # {self.home_page}'
        return line

    class Config:
        json_encoders = {
            datetime: convert_datetime_to_iso_8601_with_z_suffix
        }

    @property
    def latest_version_info(self):
        version_info = tuple(
            [
                int(num) if num.isdigit() else num
                for num in self.latest_version.replace("-", ".", 1).split(".")
            ]
        )
        return version_info

    @property
    def approved_version_info(self):
        version_info = tuple(
            [
                int(num) if num.isdigit() else num
                for num in self.approved_version.replace("-", ".", 1).split(".")
            ]
        )
        return version_info



import json
from functools import partial
from pathlib import Path
from typing import Union, Dict, Any, List


def quick_json_write(data: Union[Dict[str, Any], List[Dict[str, Any]]], file: str, output_folder: Path,
                     over_write: bool = True):
    def quick_serialize(value):
        return f'{value}'

    filename = output_folder / file

    if (filename.exists() and over_write) or not filename.exists():
        with open(filename, 'w') as json_file:
            json.dump(data, json_file, indent=4, default=quick_serialize)
        return filename


def quick_write(data: Union[Dict[str, Any], List[Dict[str, Any]]], file: str, output_subfolder: str = None,
                over_write: bool = True):
    output_folder = Path(__file__).parent.parent.parent / 'output'
    if not output_folder.exists():
        raise Exception(f'Output folder not found {output_folder}')
    if output_subfolder is None:
        folder = output_folder
    else:
        folder = output_folder / output_subfolder
    filename = quick_json_write(data, file, folder, over_write=over_write)
    return filename



from typing import Any, Dict, List

import humps

from django_scaffolding_tools.enums import ASTDataType


def build_model_serializer_template_data(parsed_django_classes: Dict[str, Any],
                                         add_source_camel_case=False) -> Dict[str, Any]:
    template_data = {'classes': list(), 'imports': ['from rest_framework import serializers']}
    for model in parsed_django_classes['classes']:
        serializer_data = dict()
        serializer_data['name'] = f'{model["name"]}Serializer(serializers.ModelSerializer)'
        serializer_data['model'] = model['name']
        serializer_data['fields'] = list()
        for attribute in model['attributes']:
            if attribute['data_type'] not in ['CONSTANT', 'UNSUPPORTED']:
                serializer_attribute = _build_serializer_field(attribute, add_source_camel_case)
                serializer_data['fields'].append(serializer_attribute)
        template_data['classes'].append(serializer_data)

    return template_data


def _build_serializer_field(attribute, add_source_camel_case):
    serializer_attribute = dict()
    serializer_attribute['name'] = attribute['name']
    serializer_attribute['add_field'] = False
    keywords = dict()
    if add_source_camel_case:
        source = humps.camelize(attribute['name'])
        if source != attribute['name']:
            serializer_attribute['source'] = source
            keywords['source'] = source
            serializer_attribute['add_field'] = True
    keyword_content = ''
    keyword_content_str = ''
    for keyword, value in keywords.items():
        keyword_content += f'{keyword}=\'{value}\', '
    if len(keyword_content) > 0:
        keyword_content_str = keyword_content[:-2]
    serializer_attribute['serializer'] = f'serializers.{attribute["data_type"]}({keyword_content_str})'
    if attribute['data_type'] == 'ForeignKey':
        keyword_content += 'read_only=True, '
        keyword_content_str = keyword_content[:-2]
        classname = attribute["arguments"][0]["value"]
        serializer_attribute['serializer'] = f'{classname}Serializer({keyword_content_str})'
        serializer_attribute['add_field'] = True
    return serializer_attribute



import os
import sys
import zipfile
from datetime import datetime
from pathlib import Path
from typing import List, Dict, Any, Tuple


def print_success(message):
    print(f"\033[92m {message}\033[00m")


def print_warning(message):
    print(f"\033[93m {message}\033[00m")


def print_error(message):
    print(f"\033[91m {message}\033[00m")


def list_all_projects(project_folder: Path) -> List[str]:
    folders = [x.path for x in os.scandir(project_folder) if x.is_dir()]
    return folders


def get_projects_envs(project_folder: Path) -> Dict[str, Any]:
    folders = list_all_projects(project_folder)
    folder_dict = dict()
    for folder in folders:
        path = Path(folder)
        envs = path / '.envs'
        if envs.exists():
            folder_dict[path.name] = {'envs': envs}
    return folder_dict


def zip_folder(zip_file: Path, folder_to_zip: Path):
    def zipdir(path, ziph):
        # ziph is zipfile handle
        for root, dirs, files in os.walk(path):
            for file in files:
                ziph.write(os.path.join(root, file),
                           os.path.relpath(os.path.join(root, file),
                                           os.path.join(path, '..')))

    with zipfile.ZipFile(zip_file, 'w', zipfile.ZIP_DEFLATED) as zipf:
        zipdir(folder_to_zip, zipf)


def backup_envs(project_folder: Path, backup_folder: Path, date_format='%Y%m%d_%H') -> Tuple[List[Path], Path]:
    project_envs_dict = get_projects_envs(project_folder)
    timestamp = datetime.now().strftime(date_format)
    b_folder = backup_folder / timestamp
    b_folder.mkdir(exist_ok=True)
    zip_list = list()
    for project, v in project_envs_dict.items():
        zip_file = b_folder / f'{project}.zip'
        zip_folder(zip_file, v['envs'])
        zip_list.append(zip_file)
    return zip_list, b_folder


if __name__ == '__main__':
    home = Path().home()
    project_folder_name = 'adelantos'
    m_folder = home / project_folder_name

    if not m_folder.exists():
        print(f'Folder {m_folder} does not exists')
        sys.exit(100)

    output_folder = home / 'Documents' / f'{project_folder_name}_envs'
    output_folder.mkdir(exist_ok=True)

    zip_files, ts_backup_folder = backup_envs(m_folder, output_folder)
    for i, zf in enumerate(zip_files, 1):
        print_success(f'{i} {zf.name}')
    print_success(f'Wrote {len(zip_files)} zip files')
    print_success(f'Output folder: {ts_backup_folder}')



from pathlib import Path

from django_scaffolding_tools.parsers import parse_file_for_ast_classes, parse_file_for_enum
from django_scaffolding_tools.django.parsers import parse_for_django_classes
from django_scaffolding_tools.writers import write_serializer_from_file, write_django_model_csv, \
    simple_write_to_excel, write_django_model_excel, write_enums


def test_write_serializers(output_folder):
    source_filename = Path(__file__).parent / 'fixtures' / 'json_data.json'
    assert source_filename.exists()

    target_file = output_folder / 'test_write_serializers.py'
    write_serializer_from_file(source_filename, target_file)
    assert target_file.exists()


def test_write_django_model_csv(output_folder, fixtures_folder):
    csv_filename = output_folder / 'model.csv'
    module_file = 'models_with_helptext.py'
    filename = fixtures_folder / module_file

    ast_module = parse_file_for_ast_classes(filename)
    django_classes = parse_for_django_classes(ast_module)
    write_django_model_csv(django_classes['classes'], csv_filename)


def test_simple_excel_writer(output_folder):
    headers = {'name': {'title': 'Name'}, 'phone_number': {'title': 'Phone number'},
               'monthly_payment': {'title': 'Monthly payment'}}
    data = [
        {'name': 'Bruce Wayne', 'phone_number': '89765400', 'monthly_payment': 234.34},
        {'name': 'Clark Kent', 'phone_number': '19765400', 'monthly_payment': 34.34},
        {'name': 'Luke Cage', 'phone_number': '99765400', 'monthly_payment': 23.34},
        {'name': 'Stephen Strange', 'phone_number': '49765400', 'monthly_payment': 234.34},
    ]
    filename = output_folder / 'excel_output.xlsx'
    simple_write_to_excel(filename, headers, data)


def test_write_django_model_excel(output_folder, fixtures_folder):
    xlsx_filename = output_folder / 'model.xlsx'
    module_file = 'models_with_helptext.py'
    filename = fixtures_folder / module_file

    ast_module = parse_file_for_ast_classes(filename)
    django_classes = parse_for_django_classes(ast_module)
    write_django_model_excel(django_classes['classes'], xlsx_filename)


def test_write_enums(fixtures_folder, output_folder):
    module_file = output_folder / '_enum.py'
    csv_file = fixtures_folder / 'g_mine_types.csv'
    results = parse_file_for_enum(csv_file, delimiter=',')
    write_enums('GoogleMimeTypes', results, module_file)
